#Sensor fix
sensors_dict = {
    "SENTINEL2_L1C": {
        "sensor_biovar": ["Cab", "Cm", "Cw", "FVC", "LAI", "laiCab", "laiCm", "laiCw"],
        "scale_factor": 0.0001,
        "bandlist": [
            "B02",
            "B03",
            "B04",
            "B05",
            "B06",
            "B07",
            "B08",
            "B8A",
            "B11",
            "B12",
        ],
    },
    "SENTINEL2_L2A": {
        "sensor_biovar": [
            "Cab",
            "Cm",
            "Cw",
            "FVC",
            "LAI",
            "laiCab",
            "laiCm",
            "laiCw",
            "CNC_Cab",
            "CNC_Cprot",
            "mangrove_Cab",
            "mangrove_Cm",
            "mangrove_Cw",
            "mangrove_LAI",
        ],
        "scale_factor": 0.0001,
        "bandlist": [
            "B02",
            "B03",
            "B04",
            "B05",
            "B06",
            "B07",
            "B08",
            "B8A",
            "B11",
            "B12",
        ],
    },
    "SENTINEL3_OLCI_L1B": {
        "sensor_biovar": ["FAPAR", "FVC", "LAI", "LCC"],
        "scale_factor": 1,
        "bandlist": [
            "B01",
            "B02",
            "B03",
            "B04",
            "B05",
            "B06",
            "B07",
            "B08",
            "B09",
            "B10",
            "B11",
            "B12",
            "B13",
            "B14",
            "B15",
            "B16",
            "B17",
            "B18",
            "B19",
            "B20",
            "B21",
        ],
    },
    "SENTINEL3_SYN_L2_SYN": {
        "sensor_biovar": ["FAPAR", "FVC"],
        "scale_factor": 1,
        "bandlist": [
          "Syn_Oa01_reflectance",
          "Syn_Oa02_reflectance",
          "Syn_Oa03_reflectance",
          "Syn_Oa04_reflectance",
          "Syn_Oa05_reflectance",
          "Syn_Oa06_reflectance",
          "Syn_Oa07_reflectance",
          "Syn_Oa08_reflectance",
          "Syn_Oa09_reflectance",
          "Syn_Oa10_reflectance",
          "Syn_Oa11_reflectance",
          "Syn_Oa12_reflectance",
          "Syn_Oa16_reflectance",
          "Syn_Oa17_reflectance",
          "Syn_Oa18_reflectance",
          "Syn_Oa21_reflectance",
          "Syn_S1N_reflectance",
          "Syn_S2N_reflectance",
          "Syn_S3N_reflectance",
          "Syn_S5N_reflectance",
          "Syn_S6N_reflectance",
        ],
    },  
}
